from django.db import models
from osinacore.models import *
import mimetypes
import os
# Create your models here.

class Visitor(models.Model):
    session_id = models.CharField(max_length=300)
    ip_address = models.CharField(max_length=300)
    country = models.CharField(max_length=15, null=True)
    url = models.URLField()
    referrer = models.URLField(null=True, blank=True)
    visit_date = models.DateTimeField(null=True)
    left_date = models.DateTimeField(null=True)

class ChatRoom(models.Model):
    name = models.CharField(max_length=300)
    created_by = models.ForeignKey(User, null=True, on_delete=models.SET_NULL, blank=True)
    date_created = models.DateTimeField()
    @property
    def last_updated(self):
        last_message = ChatMessage.objects.filter(room=self).order_by('-date_sent').first()
        if last_message:
            last_updated_time = last_message.date_sent
        else:
            last_updated_time = self.date_created

        now = datetime.now()
        if last_updated_time.date() == now.date():
            return last_updated_time.strftime('%I:%M %p')
        else:
            return last_updated_time.strftime('%d-%m-%Y')

class ChatRoomGuest(models.Model):
    room = models.OneToOneField(ChatRoom, on_delete=models.CASCADE, null=True)
    name = models.CharField(max_length=300)
    mobile_number = models.CharField(max_length=100)
    session_id = models.CharField(max_length=300)



class ChatMember(models.Model):
    member = models.ForeignKey(User, on_delete=models.CASCADE)
    room = models.ForeignKey(ChatRoom, on_delete=models.CASCADE)
    date_joined = models.DateTimeField()


class ChatProject(models.Model):
    room = models.OneToOneField(ChatRoom, on_delete=models.CASCADE)
    project = models.ForeignKey(Project, on_delete=models.CASCADE)
    public = models.BooleanField()


class ChatMessage(models.Model):
    room = models.ForeignKey(ChatRoom, on_delete=models.CASCADE, null=True)
    member = models.ForeignKey(User, null=True, on_delete=models.SET_NULL, blank=True)
    content = models.TextField(null=True, blank=True)
    date_sent = models.DateTimeField()


class ChatMessageAttachment(models.Model):
    message = models.OneToOneField(ChatMessage, on_delete=models.CASCADE)
    attachment = models.TextField()
    def is_image(self):
        mime_type, _ = mimetypes.guess_type(self.attachment)
        return mime_type and mime_type.startswith('image')
    @property
    def file_name(self):
        return os.path.basename(self.attachment)


class ChatMessageReaction(models.Model):
    message = models.ForeignKey(ChatMessage, on_delete=models.CASCADE)
    member = models.ForeignKey(User, on_delete=models.CASCADE)
    reaction = models.CharField(max_length=200)


class ChatMessageSeen(models.Model):
    message = models.ForeignKey(ChatMessage, on_delete=models.CASCADE)
    member = models.ForeignKey(User, on_delete=models.CASCADE,  null=True, blank=True)
    guest = models.ForeignKey(ChatRoomGuest, on_delete=models.CASCADE, null=True, blank=True)
    seen_date = models.DateTimeField(null=True)